/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// widget.c -- hud models

#include "gl_state.h"

gltexture_t r_translatedskin = {0, "***translateskin***"};

/*
==============================================================================

MODELS CACHED IN DISPLAY LISTS

==============================================================================
*/

qboolean TranslateMDLSkin = false;

// immediate mode drawing routine for cached brush model.  this is ONLY EVER intended to be stored
// in a display list, and should not be called during the normal refresh, as certain assumptions
// are or may be made
void Mod_DrawWidgetBrushModel (entity_t *e)
{
    model_t		*mod = e->model;
    int			i;
    int			j;
    msurface_t	*surf;
    float		boxverts[3];
    float		modHeight = mod->maxs[2] - mod->mins[2];
    float		modWidth = mod->maxs[1] - mod->mins[1];
    float		modDepth = mod->maxs[0] - mod->mins[0];
    qboolean	lumablend = false;

    // get the first surf
    surf = &mod->surfaces[mod->firstmodelsurface];

    // darken it a bit
    glColor3f (0.7f, 0.7f, 0.7f);

    allocsurfverts = 0;

    // these are normally 6 surfs with 4 verts per surf, but we can't assume that's a
    // general case...
    for (i = 0; i < mod->numsurfaces; i++, surf++)
    {
        // these shouldn't be present here but let's not mess things up anyway
        if (surf->flags & SURF_DRAWTURB) continue;
        if (surf->flags & SURF_DRAWSKY) continue;

        // reconstruct the polygon
        GL_BuildPolygonFromSurface (mod, surf);

        // hack - texture management will age these textures so ensure that they are not
        // marked as map textures to avoid this
        surf->texinfo->texture->gl_texture->flags |= TEX_NOAGE;

        if (surf->texinfo->texture->gl_texture->lumatexnum) lumablend = true;

        glBindTexture (GL_TEXTURE_2D, surf->texinfo->texture->gl_texture->gltexnum);

        glBegin (GL_TRIANGLE_FAN);

        // because we don't have polys set for anything here (so as not to mess up the vertex
        // array building) we must instead reconstruct the data from scratch)
        for (j = 0; j < surf->polys->numverts; j++)
        {
            // scale the verts so that it's sized correctly for the sbar
            boxverts[0] = ((surf->polys->verts[j].v[0] * 24) / modDepth) - 12;
            boxverts[1] = ((surf->polys->verts[j].v[1] * 24) / modWidth) - 12;
            boxverts[2] = ((surf->polys->verts[j].v[2] * 24) / modHeight) - 12;

            // draw it
            glTexCoord2fv (surf->polys->verts[j].st);
            glVertex3fv (boxverts);
        }

        glEnd ();
    }

    // back to normal colouring
    glColor3f (1, 1, 1);

    if (!lumablend) return;

    // blend mode for lumas
    glDepthMask (GL_FALSE);
    glEnable (GL_BLEND);
    glBlendFunc (GL_ONE, GL_ONE);

    // get the first surf again
    surf = &mod->surfaces[mod->firstmodelsurface];

    // luma luma luma
    for (i = 0; i < mod->numsurfaces; i++, surf++)
    {
        // these shouldn't be present here but let's not mess things up anyway
        if (surf->flags & SURF_DRAWTURB) continue;
        if (surf->flags & SURF_DRAWSKY) continue;

        if (!surf->texinfo->texture->gl_texture->lumatexnum) continue;

        glBindTexture (GL_TEXTURE_2D, surf->texinfo->texture->gl_texture->lumatexnum);

        glBegin (GL_TRIANGLE_FAN);

        // because we don't have polys set for anything here (so as not to mess up the vertex
        // array building) we must instead reconstruct the data from scratch)
        for (j = 0; j < surf->polys->numverts; j++)
        {
            // scale the verts so that it's sized correctly for the sbar
            boxverts[0] = ((surf->polys->verts[j].v[0] * 24) / modDepth) - 12;
            boxverts[1] = ((surf->polys->verts[j].v[1] * 24) / modWidth) - 12;
            boxverts[2] = ((surf->polys->verts[j].v[2] * 24) / modHeight) - 12;

            // draw it
            glTexCoord2fv (surf->polys->verts[j].st);
            glVertex3fv (boxverts);
        }

        glEnd ();
    }

    // restore the original blend modeMod_Load3DWidget
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glDisable (GL_BLEND);
    glDepthMask (GL_TRUE);
}


void Mod_DrawWidgetFrame (aliashdr_t *paliashdr, int posenum)
{
    drawvertx_t	*verts;
    int			*order;
    int			count;

    verts = (drawvertx_t *)((byte *)paliashdr + paliashdr->posedata);
    verts += posenum * paliashdr->poseverts;
    order = (int *)((byte *)paliashdr + paliashdr->commands);

    glColor3f (1, 1, 1);

    while (1)
    {
        count = *order++;

        if (!count)	break;

        if (count < 0)
        {
            count = -count;
            glBegin (GL_TRIANGLE_FAN);
        }
        else glBegin (GL_TRIANGLE_STRIP);

        do
        {
            glTexCoord2fv ((float *) order);
            glVertex3fv (verts->v);

            order += 2;
            verts++;
        }
        while (--count);

        glEnd ();
    }
}


void Mod_SetupWidgetFrame (int frame, aliashdr_t *paliashdr, gltexture_t *glt)
{
    int		pose;
    int		numposes;
    float	interval;

    if ((frame >= paliashdr->numframes) || (frame < 0))
    {
        frame = 0;
    }
    pose = paliashdr->frames[frame].firstpose;
    numposes = paliashdr->frames[frame].numposes;

    if (numposes > 1)
    {
        interval = paliashdr->frames[frame].interval;
        pose += (int)(realtime / interval) % numposes;
    }

    // this condition is only valid for the player mdl in the mplayer setup menu
    if (TranslateMDLSkin && frame)
    {
        // some vile ninja hackery going on in here...
        byte	translate[256];
        int		i;
        byte	*original;
        byte	translated[320 * 200];
        int		top = setup_top * 16;
        int		bottom = setup_bottom * 16;
        int		s = paliashdr->skinwidth * paliashdr->skinheight;

        for (i = 0; i < 256; i++)
        {
            translate[i] = i;
        }

        for (i = 0; i < 16; i++)
        {
            if (top < 128)	// the artists made some backwards ranges.  sigh.
                translate[TOP_RANGE + i] = top + i;
            else
                translate[TOP_RANGE + i] = top + 15 - i;

            if (bottom < 128)
                translate[BOTTOM_RANGE + i] = bottom + i;
            else
                translate[BOTTOM_RANGE + i] = bottom + 15 - i;
        }

        // skinnum is always 0 here
        original = (byte *) paliashdr + paliashdr->texels[0];

        for (i = 0; i < s; i += 4)
        {
            translated[i] = translate[original[i]];
            translated[i + 1] = translate[original[i + 1]];
            translated[i + 2] = translate[original[i + 2]];
            translated[i + 3] = translate[original[i + 3]];
        }

        // make a texture object the first time through here
        if (!r_translatedskin.gltexnum)
        {
            GL_MakeTextureObjects (1, (GLuint *)&r_translatedskin.gltexnum);
        }
        glBindTexture (GL_TEXTURE_2D, r_translatedskin.gltexnum);

        // upload it
        GL_LoadQuakeTexture (&r_translatedskin, "***translateskin***", translated, paliashdr->skinwidth, paliashdr->skinheight, (TEX_TEXTURE | TEX_NOGEN));

        TranslateMDLSkin = false;
    }
    else if (frame && r_translatedskin.gltexnum)
    {
        // only valid for the player mdl
        glBindTexture (GL_TEXTURE_2D, r_translatedskin.gltexnum);
    }
    else
    {
        glBindTexture (GL_TEXTURE_2D, glt->gltexnum);
    }

    // never interpolate these cos we're using temporary entities
    Mod_DrawWidgetFrame (paliashdr, pose);

    if (glt->lumatexnum)
    {
        // blend mode for lumas
        glDepthMask (GL_FALSE);
        glEnable (GL_BLEND);
        glBlendFunc (GL_ONE, GL_ONE);

        // bind
        glBindTexture (GL_TEXTURE_2D, glt->lumatexnum);

        // never interpolate these cos we're using temporary entities
        Mod_DrawWidgetFrame (paliashdr, pose);

        // restore the original blend mode
        glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glDisable (GL_BLEND);
        glDepthMask (GL_TRUE);
    }
}


// used for the player mdl
void Mod_Draw3DSkinnedWidget (char *WidgetName, int SkinNum, int framenum)
{
    model_t		*WidgetModel;
    entity_t	WidgetEntity;
    aliashdr_t	*paliashdr;

    // load the model through the standard routine
    // don;t crash if it fails, we'll just either not draw it or replace it
    // with a default model
    WidgetModel = Mod_ForName (WidgetName, false);

    // did we get a model???
    if (!WidgetModel) return;

    // only allow alias models cos brushes can cause weird shit
    if (WidgetModel->type != mod_alias) return;

    // set everything in the dummy entity to 0
    memset (&WidgetEntity, 0, sizeof (WidgetEntity));

    // set the model
    WidgetEntity.model = WidgetModel;

    // set the skin (only valid for alias mdls but does no harm)
    WidgetEntity.skinnum = SkinNum;

    // get the alias model header
    paliashdr = (aliashdr_t *) WidgetModel->extradata;

    // force a translation first time
    if (!r_translatedskin.gltexnum) TranslateMDLSkin = true;

    // draw
    Mod_SetupWidgetFrame (framenum, paliashdr, paliashdr->gl_texture[WidgetEntity.skinnum][0]);
}


// load a widget with a variable skin.
// skins must be valid for the model if it's an alias, they are ignored for brushes
int Mod_Load3DSkinnedWidget (char *WidgetName, int SkinNum, int framenum)
{
    GLuint	DisplayListNum;
    model_t *WidgetModel;
    entity_t WidgetEntity;

    // never valid for display lists
    TranslateMDLSkin = false;

    // load the model through the standard routine
    // don;t crash if it fails, we'll just either not draw it or replace it
    // with a default model
    WidgetModel = Mod_ForName (WidgetName, false);

    // did we get a model???
    if (!WidgetModel) return 0;

    // bad model type (Realm doesn't support sprites)
    if (WidgetModel->type != mod_alias && WidgetModel->type != mod_brush) return 0;

    // make a new display list object
    DisplayListNum = GL_MakeDisplayListObjects (1);
    glNewList (DisplayListNum, GL_COMPILE);

    // set everything in the dummy entity to 0
    memset (&WidgetEntity, 0, sizeof (WidgetEntity));

    // set the model
    WidgetEntity.model = WidgetModel;

    // set the skin (only valid for alias mdls but does no harm)
    WidgetEntity.skinnum = SkinNum;

    // call the appropriate drawing routine.
    // because vertex array pointers cannot be stored in display lists,
    // and the generic routines are unsuitable cos they require a worldmodel,
    // i've written some simplified immediate mode ones for this instead.
    if (WidgetModel->type == mod_alias)
    {
        // get the alias model header
        aliashdr_t *paliashdr = (aliashdr_t *) WidgetModel->extradata;

        // get the correct skin
        glBindTexture (GL_TEXTURE_2D, paliashdr->gl_texture[WidgetEntity.skinnum][0]->gltexnum);

        // draw
        Mod_SetupWidgetFrame (framenum, paliashdr, paliashdr->gl_texture[WidgetEntity.skinnum][0]);
    }
    else
    {
        // it's a brush so draw a brush
        Mod_DrawWidgetBrushModel (&WidgetEntity);
    }

    // finish the display list
    glEndList ();

    return DisplayListNum;
}


int Mod_Load3DWidget (char *WidgetName)
{
    // wouldn't overloading be so fantastic here?  oh well...
    return Mod_Load3DSkinnedWidget (WidgetName, 0, 0);
}
