/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// disable data conversion warnings

#pragma warning(disable : 4244)     // MIPS
#pragma warning(disable : 4136)     // X86
#pragma warning(disable : 4051)     // ALPHA
#pragma warning(disable : 4731)		// ASM

#if defined(_WIN32) || defined(__MINGW32__)
#include <windows.h>
#endif

#include <GL/gl.h>
#include <GL/glu.h>
#include "gl_ext.h"

void GL_BeginRendering (int *x, int *y, int *width, int *height);
void GL_EndRendering (void);

extern	float		gldepthmin, gldepthmax;
extern	int			glx, gly, glwidth, glheight;

// imports
extern float			STSinTable[SINTABLE_SIZE];
extern int				STBitMask;
extern float			STMult;
extern qboolean			skythisframe;
extern qboolean			waterthisframe;
extern qboolean			lavathisframe;
extern qboolean			telethisframe;
extern qboolean			opaquethisframe;
extern lightmapinfo_t	*lightmapinfo[MAX_LIGHTMAPS];
extern model_t			*loadmodel;
extern gltexture_t		*TextureList;

void R_TimeRefresh_f (void);
void R_ReadPointFile_f (void);
void R_AnimateLight (void);


//====================================================

// self explanatory...
extern  qboolean	TranslateMDLSkin;
extern  int			setup_top;
extern  int			setup_bottom;
extern  GLuint		spherelist;
extern  GLuint		spherelist2;
extern  int			worldtype;
extern	int			r_visframecount;	// ??? what difs?
extern	int			r_framecount;
extern  float		r_frametime;
extern  vec3_t		lightspot;
extern	vec3_t		r_modelorigin, r_entorigin;
extern	entity_t	*currententity;
extern	mplane_t	frustum[4];
extern	int		c_brush_polys, c_alias_polys;


//
// view origin
//
extern	vec3_t	vup;
extern	vec3_t	vpn;
extern	vec3_t	vright;
extern	vec3_t	r_origin;

//
// screen size info
//
extern	refdef_t	r_refdef;
extern	mleaf_t		*r_viewleaf, *r_oldviewleaf;
extern	texture_t	*r_notexture_mip;
extern	int		d_lightstylevalue[256];	// 8.8 fraction of base light value

extern	int		playertextures[16];

extern	cvar_t	r_norefresh;
extern	cvar_t	r_drawentities;
extern	cvar_t	r_drawworld;
extern	cvar_t	r_drawviewmodel;
extern	cvar_t	r_speeds;
extern	cvar_t	r_waterwarp;
extern	cvar_t	r_fullbright;
extern	cvar_t	r_lightmap;
extern	cvar_t	r_shadows;
extern	cvar_t	r_wateralpha;
extern	cvar_t	r_dynamic;
extern	cvar_t	r_novis;

extern	cvar_t	gl_clear;
extern	cvar_t	gl_cull;
extern	cvar_t	gl_poly;
extern	cvar_t	gl_smoothmodels;
extern	cvar_t	gl_affinemodels;
extern	cvar_t	gl_polyblend;
extern	cvar_t	gl_keeptjunctions;
extern	cvar_t	gl_reporttjunctions;
extern	cvar_t	gl_nocolors;
extern	cvar_t	gl_doubleeyes;
extern  cvar_t  gl_doublelight;
extern	cvar_t	gl_playermip;
extern	cvar_t  gl_lockpvs;
extern  cvar_t  gl_dot3bump;
extern  cvar_t  gl_notrans;

extern	const char *gl_vendor;
extern	const char *gl_renderer;
extern	const char *gl_version;
extern	const char *gl_extensions;

qboolean R_CullBox (vec3_t mins, vec3_t maxs);
qboolean R_CullSphere (vec3_t centre, float radius);

void R_StoreEfrags (efrag_t **ppefrag);
void R_DrawWorld (void);

void GL_Set2D (void);
void GL_TextureAging (void);
void GL_BuildLightmaps (void);
void GL_SubdivideSurface (model_t *mod, msurface_t *surf);
void GL_BuildPolygonFromSurface (model_t *mod, msurface_t *surf);

// gl_rlight.c imports
void R_LightPoint (vec3_t p, vec3_t colour);
void R_AddDynamicLights (msurface_t *surf);
void R_InitSurfStuff (void);
void R_InitBLTable (void);
void R_BuildLightMap (msurface_t *surf, qboolean initBLs);
void R_UpdateLightmapTextures (void);
void R_AnimateSurfaceLight (msurface_t *surf);
void R_ClearLightmaps (void);
void R_UploadSingleLightmap (lightmapinfo_t *lm);
void R_PushDlights (mnode_t *headnode);

void GL_CreateSurfaceLightmap (msurface_t *surf);
void GL_AllocateLightmap (int texnum, int baseintensity);
void GL_RemakeSurfaceLightmaps (void);

// gl_mesh.c imports
void GL_MakeAliasModelDisplayLists (model_t *m, aliashdr_t *hdr);

// glwarp.c imports
void R_InitSkySphere (void);
void R_DrawWaterSurfaces (void);
void R_DrawSkyPickup (qboolean skydraw);

void R_TranslatePlayerSkin (int playernum);
void R_AddStain (float *org, float red, float green, float blue, float radius);
void R_ParticleSprite (int color, float radius, vec3_t origin, pdrawstyle_t ptype);

#ifndef _WIN32
#define APIENTRY /* */
#endif

